//Author: Andrew Shorten
//Date: March 2009
//Purpose: Provide double buffered c access to the xps-tft module

//Define the screen resolution
#define VGA_X_RES 640
#define VGA_Y_RES 480

//The struct responsible for representing a single vga screen
//In a regular setup, two of these structs are used to perform
//double buffering
struct Screen
{
	unsigned int pixel_data[491520]; // 0x78000
};

//The struct responsible for interfacing with a vga controller module
//Takes care of double buffering
struct Buffered_screen
{
	struct Screen* screen_0;
	struct Screen* screen_1;
	unsigned int* control;
	unsigned int displayed;
};

//Writes the given pixel value to the given screen at the given x-y location
void write_pixel(struct Screen* in, unsigned int value, unsigned int x, unsigned int y)
{
	in->pixel_data[(y * 1024) + x] = value;
}

//Writes the given RGB pixel value to the given screen at the given x-y location
void write_pixel_rgb(struct Screen* in, unsigned int R, unsigned int G, unsigned int B, unsigned int x, unsigned int y)
{
		in->pixel_data[(y * 1024) + x] = (R * 65536) + (G * 256) + B;
}

//Clears the given screen to the given pixel value
void clear_screen(struct Screen* in, unsigned int value)
{
	int i, j;
	
	//Loop through all pixels and set them to "value"
	for(j = 0; j < VGA_Y_RES; j++)
	{
		for(i = 0; i < VGA_X_RES; i++)
		{
			in->pixel_data[(j * 1024) + i] = value;
		}
	}

}

//Initializes the given Buffered_screen struct with the given screen and xps_tft control pointer information
//The control pointer points to the first memory location of the xps_tft's PLB slave interface and can be used
//to switch what place in memory the module is reading from
void init_buffered_screen(struct Buffered_screen* in, struct Screen* screen_a, struct Screen* screen_b, unsigned int* control)
{
	//copy the screen pointers
	in->screen_0 = screen_a;
	in->screen_1 = screen_b;
	
	//clear the given screens
	clear_screen(screen_a, 0);
	clear_screen(screen_b, 0);
	
	//Set up the doubl buffering state
	in->displayed = 0;
	
	//copy the vga-controllers access pointer
	in->control = control;
	(*(in->control)) = (unsigned int)screen_a;
}

//Makes the VGA controller swap between the two different vga screens
void swap(struct Buffered_screen* in)
{
	//Inspect which screen is beeing displayed
	if(in->displayed == 0)
	{
		//Switch which screen is beeing displayed
		in->displayed = 1;
		
		//Force the vga controller to switch to the new screen
		(*(in->control)) = (unsigned int)(in->screen_1);
	}
	else
	{
		//Switch which screen is beeing displayed
		in->displayed = 0;
		
		//Force the vga controller to switch to the new screen
		(*(in->control)) = (unsigned int)(in->screen_0);
	}
}
/*
void setBuffer(struct Buffered_screen *in, struct Screen* curr) {
	//Inspect which screen is beeing displayed
	if(in->displayed == 0)
	{
		//Switch which screen is beeing displayed
		in->screen_1 = curr;
		in->displayed = 1;
		
		//Force the vga controller to switch to the new screen
		(*(in->control)) = (int)(in->screen_1);
	}
	else
	{
		//Switch which screen is beeing displayed
		in->screen_0 = curr;
		in->displayed = 0;
		
		//Force the vga controller to switch to the new screen
		(*(in->control)) = (int)(in->screen_0);
	}
}*/

//Returns the screen that is currently beeing displayed
struct Screen* get_displayed(struct Buffered_screen* in)
{
	//Check which screen is beeing displayed and returns the appropriate one
	if(in->displayed == 0)
	{
		return in->screen_0;
	}
	else
	{
		return in->screen_1;
	}
}

//Returns the screen that is *not* currently beeing displayed
struct Screen* get_undisplayed(struct Buffered_screen* in)
{
	//Check which screen is beeing displayed and returns the appropriate one
	if(in->displayed == 0)
	{
		return in->screen_1;
	}
	else
	{
		return in->screen_0;
	}
}

//Gets the pixel value at the given x-y location of the given vga screen
unsigned int get_pixel(struct Screen* in, unsigned int x, unsigned int y)
{
	return in->pixel_data[(y * 1024) + x];
}
